@@ -24,9 +24,11 @@
 #ifndef GI_PARAM_H_
 #define GI_PARAM_H_
 
+#include <config.h>
+
 #include <glib-object.h>
 
-#include "gjs/jsapi-wrapper.h"
+#include <js/TypeDecls.h>
 
 #include "gjs/macros.h"
 
