@@ -24,9 +24,11 @@
 #ifndef GI_REPO_H_
 #define GI_REPO_H_
 
+#include <config.h>
+
 #include <girepository.h>
 
-#include "gjs/jsapi-wrapper.h"
+#include <js/TypeDecls.h>
 
 #include "gjs/macros.h"
 #include "util/log.h"
