@@ -24,10 +24,12 @@
 #ifndef GJS_JSAPI_CLASS_H_
 #define GJS_JSAPI_CLASS_H_
 
+#include <config.h>
+
 #include <glib-object.h>
 #include <glib.h>
 
-#include "gjs/jsapi-wrapper.h"
+#include <js/TypeDecls.h>
 
 #include "gi/wrapperutils.h"  // IWYU pragma: keep
 #include "gjs/global.h"       // IWYU pragma: keep
@@ -83,7 +85,6 @@ bool gjs_define_property_dynamic(JSContext       *cx,
     }                                                                          \
     GJS_ALWAYS_INLINE GJS_USE static inline type* priv_from_js(                \
         JSContext* cx, JS::HandleObject obj) {                                 \
-        JSAutoRequest ar(cx);                                                  \
         return static_cast<type*>(                                             \
             JS_GetInstancePrivate(cx, obj, &klass, nullptr));                  \
     }                                                                          \
