@@ -21,10 +21,10 @@
  * IN THE SOFTWARE.
  */
 
-#include <config.h>  // for ENABLE_PROFILER, HAVE_SYS_SYSCALL_H, HAVE_UNISTD_H, HAVE_SYS_SIGNAL_H
+#include <config.h>  // for ENABLE_PROFILER, HAVE_SYS_SYSCALL_H, HAVE_UNISTD_H
 
-#ifdef HAVE_SYS_SIGNAL_H
-#	include <sys/signal.h>  // for siginfo_t, sigevent, ...
+#ifndef HAVE_SIGNAL_H
+#    include <signal.h>  // for siginfo_t, sigevent, sigaction, SIGPROF, ...
 #endif
 
 #include <glib-object.h>
@@ -33,7 +33,6 @@
 #ifdef ENABLE_PROFILER
 #    include <alloca.h>
 #    include <errno.h>
-#    include <signal.h>  // for sigaction, SIGPROF, sigemptyset
 #    include <stddef.h>  // for size_t
 #    include <stdint.h>
 #    include <stdio.h>      // for sscanf
@@ -54,8 +53,8 @@
 #    include <sysprof-capture.h>
 #endif
 
-#include "gjs/jsapi-wrapper.h"  // IWYU pragma: keep
-#include "js/ProfilingStack.h"  // for EnableContextProfilingStack, ...
+#include <js/ProfilingStack.h>  // for EnableContextProfilingStack, ...
+#include <js/TypeDecls.h>
 
 #include "gjs/context.h"
 #include "gjs/jsapi-util.h"
@@ -104,7 +103,7 @@ struct _GjsProfiler {
      * information while executing. We will look into this during our
      * SIGPROF handler.
      */
-    PseudoStack stack;
+    ProfilingStack stack;
 
     /* The context being profiled */
     JSContext *cx;
@@ -271,7 +270,7 @@ _gjs_profiler_free(GjsProfiler *self)
     if (self->fd != -1)
         close(self->fd);
 
-    self->stack.~PseudoStack();
+    self->stack.~ProfilingStack();
 #endif
     g_free(self);
 }
@@ -329,7 +328,7 @@ static void gjs_profiler_sigprof(int signum G_GNUC_UNUSED, siginfo_t* info,
         static_cast<SysprofCaptureAddress*>(alloca(sizeof *addrs * depth));
 
     for (uint32_t ix = 0; ix < depth; ix++) {
-        js::ProfileEntry& entry = self->stack.entries[ix];
+        js::ProfilingStackFrame& entry = self->stack.frames[ix];
         const char *label = entry.label();
         const char *dynamic_string = entry.dynamicString();
         uint32_t flipped = depth - 1 - ix;
