@@ -1,3 +1,5 @@
+/* eslint-disable no-restricted-properties */
+
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
 const Signals = imports.signals;
@@ -5,7 +7,7 @@ const Signals = imports.signals;
 const Foo = new Lang.Class({
     Name: 'Foo',
     Implements: [Signals.WithSignals],
-    _init: function () {},
+    _init() {},
 });
 
 describe('Legacy object with signals', function () {
@@ -28,17 +30,17 @@ function testSignals(klass) {
 
     it('calls a signal handler when a signal is emitted', function () {
         foo.connect('bar', bar);
-        foo.emit('bar', "This is a", "This is b");
+        foo.emit('bar', 'This is a', 'This is b');
         expect(bar).toHaveBeenCalledWith(foo, 'This is a', 'This is b');
     });
 
     it('does not call a signal handler after the signal is disconnected', function () {
         let id = foo.connect('bar', bar);
-        foo.emit('bar', "This is a", "This is b");
+        foo.emit('bar', 'This is a', 'This is b');
         bar.calls.reset();
         foo.disconnect(id);
         // this emission should do nothing
-        foo.emit('bar', "Another a", "Another b");
+        foo.emit('bar', 'Another a', 'Another b');
         expect(bar).not.toHaveBeenCalled();
     });
 
@@ -107,7 +109,7 @@ function testSignals(klass) {
             foo.connect('bar', bar);
             foo.connect('bar', bar2);
             GLib.test_expect_message('Gjs', GLib.LogLevelFlags.LEVEL_WARNING,
-                                     'JS ERROR: Exception in callback for signal: *');
+                'JS ERROR: Exception in callback for signal: *');
             foo.emit('bar');
         });
 
@@ -118,7 +120,7 @@ function testSignals(klass) {
 
         it('does not disconnect the callback', function () {
             GLib.test_expect_message('Gjs', GLib.LogLevelFlags.LEVEL_WARNING,
-                                     'JS ERROR: Exception in callback for signal: *');
+                'JS ERROR: Exception in callback for signal: *');
             foo.emit('bar');
             expect(bar).toHaveBeenCalledTimes(2);
             expect(bar2).toHaveBeenCalledTimes(2);
