@@ -54,7 +54,7 @@ main(int argc, char **argv)
         g_irepository_prepend_search_path(g_getenv("TOP_BUILDDIR"));
     } else {
         g_irepository_prepend_search_path(INSTTESTDIR);
-        g_irepository_prepend_library_path(PKGLIBDIR);
+        g_irepository_prepend_library_path(INSTTESTDIR);
     }
 
     const char *coverage_prefix = g_getenv("GJS_UNIT_COVERAGE_PREFIX");
