@@ -20,10 +20,17 @@
  * IN THE SOFTWARE.
  */
 
+#include <config.h>
+
 #include <cairo.h>
 #include <glib.h>
 
-#include "gjs/jsapi-wrapper.h"
+#include <js/CallArgs.h>
+#include <js/Class.h>
+#include <js/PropertySpec.h>
+#include <js/RootingAPI.h>
+#include <js/TypeDecls.h>
+#include <jsapi.h>  // for JS_NewObjectWithGivenProto
 
 #include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util-args.h"
@@ -133,7 +140,7 @@ gjs_cairo_solid_pattern_from_pattern(JSContext       *context,
         return nullptr;
     }
 
-    gjs_cairo_pattern_construct(context, object, pattern);
+    gjs_cairo_pattern_construct(object, pattern);
 
     return object;
 }
