@@ -20,10 +20,14 @@
  * IN THE SOFTWARE.
  */
 
+#include <config.h>
+
 #include <cairo-features.h>  // for CAIRO_HAS_PDF_SURFACE, CAIRO_HAS_PS_SURFA...
 #include <cairo.h>
 
-#include "gjs/jsapi-wrapper.h"
+#include <js/RootingAPI.h>
+#include <js/TypeDecls.h>
+#include <jsapi.h>  // for JS_NewPlainObject
 
 #include "gjs/jsapi-util.h"
 #include "modules/cairo-module.h"  // IWYU pragma: keep
